@@ -1,5 +1,29 @@
 pub mod dp {
     //! This is a module for dynamic programming.
+
+    use itertools::structs::Combinations;
+
+    struct MultiCombination<I: Iterator> {
+        combs: Vec<Combinations<I>>,
+    }
+    
+    impl<I> Iterator for MultiCombination<I> 
+        where I: Iterator,
+        I::Item: Clone  
+    {
+        type Item = Vec<I::Item>;
+    
+        fn next(&mut self) -> Option<Self::Item> {
+            for comb in &mut self.combs {
+                if let Some(elt) = comb.next() {
+                    return Some(elt);
+                } else {
+                    continue
+                }
+            }
+            None
+        }
+    }
     use std::collections::HashMap;
     use rayon::prelude::*;
     use std::sync::{Arc, Mutex};
@@ -61,62 +85,51 @@ pub mod dp {
         use std::cmp::min;
         // https://stackoverflow.com/questions/43078142/subset-sum-with-negative-values-in-c-or-c
         // Find a subset even if an array contains negative values.
-        let mut b: Vec<u32> = Vec::with_capacity(arr.len());
         let answer: Arc<Mutex<Vec<Vec<i32>>>> = Arc::new(Mutex::new(vec![]));
         if arr.iter().min().unwrap() >= &0 && value > 0 {
-            for i in arr {
-                b.push(i as u32);
-            }
-            let result = find_subset_fast_only_positive(&b, value as usize, max_length);
+            let result = find_subset_fast_only_positive(&arr.iter().map(|e| *e as u32).collect::<Vec<u32>>(), value as usize, max_length);
             for i in result {
-                let mut tempvec = Vec::with_capacity(i.len());
-                for j in i {
-                    tempvec.push(j as i32);
-                }
-                answer.lock().unwrap().push(tempvec)
+                answer.lock().unwrap().push(i.iter().map(|e| *e as i32).collect::<Vec<i32>>())
             }
             return answer.lock().unwrap().to_vec();
         } else {
             let length = arr.len();
-            let offset: u32 =
-                (max(arr.iter().min().unwrap().abs() + 1, min(value, 0).abs() + 1)) as u32;
-            for i in arr {
-                b.push((i + offset as i32) as u32);
-            }
+            let offset: i32 =
+                (max(arr.iter().min().unwrap().abs() + 1, min(value, 0).abs() + 1)) as i32;
             // We will transform the array into a new array whose elements are all positive.
             // And check if the transformed sum of the result of the new array is equal to the target value.
             // If we find the sum is the same as the target, we will return the result.
             (1..min(length, max_length) + 1).into_par_iter().for_each(|i| {
-                let result = find_subset_fast_only_positive(
-                    &b,
-                    (value + i as i32 * offset as i32) as usize,
+                let result = _find_subset_fast_only_positive(
+                    &arr.iter().map(|e| (e + offset) as u32).collect::<Vec<u32>>(),
+                    (value + i as i32 * offset) as usize,
                     max_length,
                 );
                 for res in result {
                     let mut tempsum: i32 = 0;
                     let mut new_res: Vec<i32> = Vec::with_capacity(res.len());
-                    for j in res {
-                        tempsum += j as i32 - offset as i32;
-                        new_res.push(j as i32 - offset as i32);
+                    for j in &res {
+                        tempsum += *j as i32 - offset;
+                        new_res.push(*j as i32 - offset);
                     }
-                    if tempsum == value as i32 {
+                    if tempsum == value {
                         answer.lock().unwrap().push(new_res);
                     }
                 }
             });
-            return answer.lock().unwrap().to_vec();
+            return vector_sorter(answer.lock().unwrap().to_vec());
         };
     }
 
     fn rec(
-        dp: &Vec<Vec<i32>>,
+        dp: &Vec<u32>,
         arr: &Vec<u32>,
         i: usize,
         j: usize,
         route: &mut Vec<u32>,
         answer: &mut Vec<Vec<u32>>,
-        a_min: &u32,
         max_length: usize,
+        collen: usize
     ) {
         // This code is mostly copied from https://drken1215.hatenablog.com/entry/2019/12/17/190300
         // We follow the dp table backward to find combinations of subsets.
@@ -135,11 +148,11 @@ pub mod dp {
             return;
         }
 
-        if dp[i - 1][j] != 0 {
-            rec(dp, arr, i - 1, j, route, answer, a_min, max_length);
+        if dp[(i - 1) * collen + j] != 0 {
+            rec(dp, arr, i - 1, j, route, answer, max_length, collen);
         }
 
-        if j as i32 - arr[i - 1] as i32 >= 0 && dp[i - 1][j - arr[i - 1] as usize] != 0 {
+        if j as i32 - arr[i - 1] as i32 >= 0 && dp[(i - 1) * collen + j - arr[i - 1] as usize] != 0 {
             // Choose this element as arr candidate for an answer.
             route.push(arr[i - 1]);
             rec(
@@ -149,8 +162,8 @@ pub mod dp {
                 j - arr[i - 1] as usize,
                 route,
                 answer,
-                a_min,
                 max_length,
+                collen
             );
             // Remove this element after we reach i == 0 regardless of whether we reach j == 0.
             route.pop();
@@ -267,30 +280,45 @@ pub mod dp {
         arr: &Vec<u32>,
         value: usize,
         max_length: usize,
+    ) -> Vec<Vec<u32>> {
+        let answer = _find_subset_fast_only_positive(arr, value, max_length);
+        vector_sorter(answer)
+    }
+
+    fn _find_subset_fast_only_positive(
+        arr: &Vec<u32>,
+        value: usize,
+        max_length: usize,
     ) -> Vec<Vec<u32>> {
         // dp is a table that stores the information of subset sum.
         // dp[i][j] is the number of ways to make sum j with i element.
         // We follow from the start of this table.
-        let mut dp: Vec<Vec<i32>> = vec![vec![0; value + 1]; arr.len() + 1];
-        dp[0][0] = 1;
+        // let mut dp: Vec<Vec<i32>> = vec![vec![0; value + 1]; arr.len() + 1];
+        let collen = value + 1;
+        let mut dp: Vec<u32> = vec![0; (value + 1) * (arr.len() + 1)];
+        dp[0] = 1;
 
-        let (j_indexes, a_min) = filter_j_idx(value, arr);
-        for i in 0..arr.len() {
+        let (j_indexes, _a_min) = filter_j_idx(value, arr);
+        for (i, v_u32) in arr.iter().enumerate() {
+            let v_usize = *v_u32 as usize;
             for j in &j_indexes {
                 // If we don't choose to select an element to sum,
                 // the ways to make a sum are the same as with the previous element.
-                dp[i + 1][*j] += dp[i][*j];
+                // dp[i + 1][*j] += dp[i][*j];
+                dp[(i + 1) * collen + *j] += dp[i * collen + *j]; 
 
                 // Skip if j + the element is larger than the target value.
-                if *j as u32 + arr[i] < value as u32 + 1 {
+                if *j + v_usize < value + 1 {
                     // This means we find another way to make sum j with i elements
                     // when we choose this element as an element to sum.
-                    dp[i + 1][j + arr[i] as usize] += dp[i][*j];
+                    // dp[i + 1][j + arr[i] as usize] += dp[i][*j];
+                    dp[(i + 1) * collen + j + v_usize] += dp[i * collen + *j]; 
+
                 }
-            }
+            };
         }
         let a_length: usize = arr.len();
-        let mut route: Vec<u32> = Vec::with_capacity(max_length);
+        let mut route: Vec<u32> = vec![];
         let mut answer: Vec<Vec<u32>> = vec![];
 
         rec(
@@ -300,10 +328,10 @@ pub mod dp {
             value,
             &mut route,
             &mut answer,
-            &a_min,
             max_length,
+            collen
         );
-        vector_sorter(answer)
+        answer
     }
 
     fn vec_remove(arr: &mut Vec<i32>, v: i32) {
@@ -390,7 +418,6 @@ pub mod dp {
         }
         answer2.sort();
         answer2.dedup();
-        // answer2.sort_by_key(|x| sequence_matcher_sort(x.to_vec()));
         if answer2.len() == 0 {
             println!("Can't find any combination.");
         }
@@ -406,12 +433,12 @@ pub mod dp {
         max_target_length: usize,
         hashmap_fs: &mut Arc<Mutex<HashMap<(Vec<i32>, i32), Vec<Vec<i32>>>>>,
         n_candidates: usize,
-    ) {
+    ) -> (){
         use itertools::Itertools;
+        use std::cmp::min;
         if answer.lock().unwrap().len() >= n_candidates {
             return;
         }
-
         if keys.len() == 0 && targets.len() == 0 {
             group.sort_by_key(|k| k.0.iter().sum::<i32>());
             group.sort_by_key(|k| k.0.len());
@@ -425,37 +452,46 @@ pub mod dp {
         if (keys.len() == 0 && targets.len() > 0) || (keys.len() > 0 && targets.len() == 0) {
             return;
         }
-        (0..keys.len()).powerset().filter(|x| x.len() <= max_key_length).par_bridge().for_each(|i| {
-            let keys2 = keys.clone();
-            let mut targets2 = targets.clone();
-            let group2 = group.clone();
+        targets.sort();
+        let mut combs = vec![];
+        for i in 1..min(max_key_length, keys.len()) + 1{
+            combs.push((0..keys.len()).into_iter().combinations(i))
+        };
+        let mc = MultiCombination{
+            combs: combs,
+        };
+        mc.par_bridge().for_each(|i| {
             let mut sum_key = 0;
             let mut vec_key = vec![];
             for j in i.iter() {
-                sum_key += keys2[*j];
-                vec_key.push(keys2[*j].clone());
+                sum_key += keys[*j];
+                vec_key.push(keys[*j].clone());
             }
             vec_key.sort();
-            if sum_key > targets2.iter().sum() {
+            if sum_key > targets.iter().sum() {
                 return;
             }
-            if targets2.iter().max().unwrap() == &0 {
+            if sum_key < *targets.iter().min().unwrap() {
                 return;
             }
-            targets2.sort();
-            let set_ = match hashmap_fs.try_lock() {
-                Ok(mut v) => v.entry((targets2.clone(), sum_key))
-                    .or_insert(find_subset(targets2.clone(), sum_key, max_target_length))
-                    .clone(),
-                Err(_) => find_subset(targets2.clone(), sum_key, max_target_length)
+            if targets.iter().max().unwrap() == &0 {
+                return;
+            }
+            let mut set_ = match hashmap_fs.try_lock() {
+                Ok(mut v) => {v.entry((targets.clone(), sum_key))
+                    .or_insert(find_subset(targets.clone(), sum_key, max_target_length))
+                    .clone()},
+                Err(_) => {find_subset(targets.clone(), sum_key, max_target_length)}
             };
             if set_.len() == 0 {
                 return;
             }
+            set_.sort();
+            set_.dedup();
             set_.par_iter().for_each(|set| {
-                let mut keys3 = keys2.clone();
-                let mut targets3 = targets2.clone();
-                let mut group3 = group2.clone();
+                let mut keys3 = keys.clone();
+                let mut targets3 = targets.clone();
+                let mut group3 = group.clone();
                 group3.push((vec_key.clone(), set.clone()));
                 for j in set.clone() {
                     vec_remove(&mut targets3, j);
@@ -473,8 +509,9 @@ pub mod dp {
                     &mut hashmap_fs.clone(),
                     n_candidates
                 );
-            })
+            });
         });
+        ()
     }
 
     #[test]
@@ -495,26 +532,27 @@ pub mod dp {
             2,
             200
         );
-        assert_eq!(answer.len(), 197);
+        assert_eq!(answer.len(), 195);
         assert_eq!(
             answer[0],
             vec![
-                (vec![], vec![-700, 700]),
-                (vec![200, 300], vec![500]),
-                (vec![100, 500], vec![600]),
-                (vec![-700, 400, 600], vec![300]),
+                (vec![-700], vec![-700]),
+                (vec![100, 200], vec![300]),
+                (vec![300, 400], vec![700]),
+                (vec![500, 600], vec![500, 600]),
                 (vec![800, 900, 1000], vec![2700]),
             ]
         );
 
         let answer = sequence_matcher(&mut vec![9, 0, 1, 7, 1], &mut vec![7, 2, 8, 0, 1], 3, 2, 100);
-        assert_eq!(answer.len(), 37);
+        assert_eq!(answer.len(), 24);
         assert_eq!(
             answer[0],
             vec![
-                (vec![], vec![0]),
-                (vec![0, 9], vec![1, 8]),
-                (vec![1, 1, 7], vec![2, 7]),
+                (vec![0], vec![0]),
+                (vec![1], vec![1]),
+                (vec![7], vec![7]),
+                (vec![1, 9], vec![2, 8]),
             ]
         );
 
